/*
 * rocket2.h --- Include file for rocket2.c, the Comtrol RocketPort2 Linux driver.
 *
 * Copyright 2004-2011, Comtrol Corporation.    All Rights Reserved.
 *
 */

#ifndef _ROCKET2_H
#define _ROCKET2_H

#include <asm/io.h>
#include <asm/byteorder.h>
#include <linux/serial.h>
#include <linux/version.h>
#include <linux/termios.h>

#define ROCKET_VERSION "1.22"
#define ROCKET_DATE "11-Aug-2011"

#define RP2_MAJOR_NUMBER  47

#define Bit(n) (1<<(n))

/*  If defined, uses Linux xx_USE_COUNT mechanism, else uses our own mechanism  */
#define USE_USE_COUNT

#define FIFO_SIZE     512           /*  FIFO size for both TX and RX FIFO's */
#define XMIT_BUF_SIZE PAGE_SIZE     /* The size of the xmit buffer is 1 page */

/* number of characters left in xmit buffer before we ask for more */
#define WAKEUP_CHARS 256

#define MAX_NUMBEROF_BOARDS 4   //according to our documentation (README, etc)
#define MAX_PORTS_PER_BOARD 32
#define MAX_NUMBEROF_PORTS  (MAX_NUMBEROF_BOARDS * MAX_PORTS_PER_BOARD)

#define RPORT_MAGIC 0x625001

/*
 *  Bit defines used in TTY processing of received bytes.  There are 2 control words:
 *  - ChP->ignore_status -  bytes with the attribute are dropped
 *  - ChP->report_status -  bytes with the attribute are reported in the flag_buf as bad.  Byte saved.
 */
#define U_PARITY   Bit(0)
#define U_FRAMING  Bit(1)
#define U_BREAK    Bit(2)
#define U_OVERRUN  Bit(3)

/*  Flow control defines  */
#define FLOW_CONTROL_NONE    0
#define FLOW_CONTROL_RTSCTS  1
#define FLOW_CONTROL_XONXOFF 2
#define FLOW_CONTROL_DTRDSR  3


// driver-specific values used with serial_struct flags field

#define ASYNC_SPD_WARP13       (ASYNC_SPD_HI|ASYNC_SPD_VHI|ASYNC_SPD_SHI)

#if !defined(ASYNCB_LAST_USER)
# define ASYNCB_LAST_USER 15
#endif

#define ROCKETB_RTS_TOGGLE      (ASYNCB_LAST_USER+1)
#define ROCKETB_IS_SMPTE        (ASYNCB_LAST_USER+2)
#define ROCKETB_MODE            (ASYNCB_LAST_USER+3)
#define ROCKETB_MODE_WIDTH      3
#define ROCKETB_LAST            (ROCKETB_MODE+ROCKETB_MODE_WITDH-1)

#define ROCKET_RTS_TOGGLE     (1U << ROCKETB_RTS_TOGGLE)
#define ROCKET_IS_SMPTE       (1U << ROCKETB_IS_SMPTE)

#define ROCKET_MODE_MASK      ((~((~0U)<<ROCKETB_MODE_WIDTH)) << ROCKETB_MODE)
#define ROCKET_MODE_RS232     (0 << ROCKETB_MODE)
#define ROCKET_MODE_RS422     (1 << ROCKETB_MODE)
#define ROCKET_MODE_RS485     (2 << ROCKETB_MODE) // 485 2-wire
#define ROCKET_MODE_RS485FDS  (3 << ROCKETB_MODE) // 485 4-wire slave
#define ROCKET_MODE_RS485FDM  (4 << ROCKETB_MODE) // 485 4-wire master

#define ROCKET_FLAGS  (ROCKET_MODE_MASK | ROCKET_RTS_TOGGLE)

// flags users can change
#define ROCKET_USR_MASK         (ASYNC_USR_MASK | ROCKET_FLAGS)
#define ROCKET_ALLFLAGS            (ASYNC_FLAGS | ROCKET_FLAGS)

#ifndef MIN
#define MIN(a,b)        ((a) < (b) ? (a) : (b))
#endif

/*  Some TTY defines  */
#define VERSION_CODE(vers,rel,seq) ( ((vers)<<16) | ((rel)<<8) | (seq) )
//^moved here from rocket2.c, since we added check below:

#if LINUX_VERSION_CODE < VERSION_CODE(2,6,27)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif

#if LINUX_VERSION_CODE > VERSION_CODE(2,5,68)
#define TTY_GET_LINE(t) t->index
#define TTY_DRIVER_MINOR_START(t) t->driver->minor_start
#define TTY_DRIVER_SUBTYPE(t) t->driver->subtype
#define TTY_DRIVER_NAME(t) t->driver->name
#define TTY_DRIVER_NAME_BASE(t) t->driver->name_base
#if LINUX_VERSION_CODE < VERSION_CODE(2,6,26)
#define TTY_DRIVER_FLUSH_BUFFER_EXISTS(t) t->driver->flush_buffer
#define TTY_DRIVER_FLUSH_BUFFER(t) t->driver->flush_buffer(t)
#define TTY_LDISC_FLUSH_BUFFER_EXISTS(t) t->ldisc.flush_buffer
#define TTY_LDISC_FLUSH_BUFFER(t) t->ldisc.flush_buffer(t)
#else           // implies kernel >= 2,6,26
//do this instead. the new api validates the pointer for us
#define TTY_DRIVER_FLUSH_BUFFER_EXISTS(t) 1
#define TTY_DRIVER_FLUSH_BUFFER(t) tty_driver_flush_buffer(t)
#define TTY_LDISC_FLUSH_BUFFER_EXISTS(t) 1
#define TTY_LDISC_FLUSH_BUFFER(t) tty_ldisc_flush(t)
#endif          //kernel < 2,6,26
#else           // implies kernel <= 2,5,68
#define TTY_GET_LINE(t) minor(t->device) - TTY_DRIVER_MINOR_START(t)
#define TTY_DRIVER_MINOR_START(t) t->driver.minor_start
#define TTY_DRIVER_SUBTYPE(t) t->driver.subtype
#define TTY_DRIVER_NAME(t) t->driver.name
#define TTY_DRIVER_NAME_BASE(t) t->driver.name_base
#define TTY_DRIVER_FLUSH_BUFFER_EXISTS(t) t->driver.flush_buffer
#define TTY_DRIVER_FLUSH_BUFFER(t) t->driver.flush_buffer(t)
#endif //  LINUX_VERSION > 2.5.68


/*
 *   Rocketport ioctls
 */
#define RCKP_GET_STRUCT         0x00525001
#define RCKP_GET_CONFIG         0x00525002
#define RCKP_SET_CONFIG         0x00525003
#define RCKP_GET_PORTS          0x00525004
#define RCKP_RESET_RM2          0x00525005
#define RCKP_GET_VERSION        0x00525006
#define RP2_RESET_USE_COUNT     0x00525007

/*  Model Information Struct */
typedef struct {
    unsigned long model;
    char modelString[80];
    unsigned long numPorts;
    int startingPortNumber;
} rocketModel_t;

typedef struct {
    unsigned long model;
    char modelString[80];
    unsigned long numPorts;
    int has_2nd_asic;
    int is_smpte;
} modelInfo_t;

struct rocket_version {
    char rocket_version[32];
    char rocket_date[32];
    char reserved[64];
};

// Channel level information structure
typedef struct {
    void *pBaseRegister;          // Base ptr to port registers
    void *ucodeAddr;              // Base ptr to the channel's Receive Processor microcode.

    int channelNumber;            // The ASIC channel number (0-31)
    int portNumber;               // The port number /dev/ttyRPx (x) number assigned to the TTY layer
    int xmit_flag;                // Flag indicating there is data to be transmitted

    // stuff correspondig to serial_struct fields used by setserial/setinfinity
    int flags;
    int closing_wait;             // milliseconds
    int close_delay;              // milliseconds
    unsigned long baud_base;
    unsigned long custom_divisor;

    int magic;
    int count;                    // How may processes have this port open
    int blocked_open;             // Flag indicating port is blocking, waiting to open
    struct tty_struct *rptty;
    int monitorCdChange;          // A TTY thing, the flag CLOCAL controls whether delta CD's is acted upon

    /*  The XMIT buffer */
    unsigned char *xmit_buf;
    int xmit_head;
    int xmit_tail;
    int xmit_cnt;                // Number of bytes in the XMIT buffer
    int xmit_fifo_room;          // Amount of room in the TX FIFO

    int cd_status;

    int ignore_status;           // TTY byte processing, see above
    int report_status;

    int cps;                     // Character per second value, used when waiting for a drain
    struct termios normal_termios;

#if LINUX_VERSION_CODE < VERSION_CODE(2,6,10)
    unsigned char *tmp_buf;
    struct semaphore tmp_buf_sem;
#endif

#ifdef DECLARE_WAITQUEUE
    wait_queue_head_t open_wait;
    wait_queue_head_t close_wait;
    wait_queue_head_t delta_msr_wait;  //  Queue of processes waiting on a MSR change
#else
    struct wait_queue *open_wait;
    struct wait_queue *close_wait;
    struct wait_queue *delta_msr_wait;
#endif
    struct async_icount icount;  // count of MSR line changes
    int am_delta_msr_waiting;    // Flag, indicating a process has called TIOCMIWAIT and is sleeping */
    unsigned long last_msr;      // The last MSR values read

    spinlock_t slock;

} CHANNEL_T;


/* Controller level information structure */
typedef struct {
    int controllerNumber;                         /*  Controller number (0-n) */
    int numPorts;                                 /*  Number of channels (serial ports) */
    int startingPort;                             /*  Starting port number ttyRPx (x) assigned by the TTY layer */
    rocketModel_t rocketModel;

    /*  Model information */
    int clockPrescale;
    int has_2nd_asic;                             /*  Set if board has 2 Unity ASICs */

    CHANNEL_T channel[MAX_PORTS_PER_BOARD];       /*  One channel structure per serial port */

    void *pBasePciRegion0;               //  Base memory address region 0
    void *pBasePciMemory;                //  Base memory address region 1
} CONTROLLER_T;

/*  Debugging MACROS  */

//#define ROCKET_DEBUG_ALL

#ifdef ROCKET_DEBUG_ALL

#define ROCKET_DEBUG_OPEN
#define ROCKET_DEBUG_CLOSE
#define ROCKET_DEBUG_INTR
#define ROCKET_DEBUG_IO
#define ROCKET_DEBUG_WRITE
#define ROCKET_DEBUG_READ
#define ROCKET_DEBUG_FLOW
#define ROCKET_DEBUG_WAIT_UNTIL_SENT
#define ROCKET_DEBUG_DELTA_MSR

#endif // ROCKET_DEBUG_ALL

#ifdef ROCKET_DEBUG_OPEN
#  define DOPEN(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DOPEN(fmt, args...)
#endif

#ifdef ROCKET_DEBUG_CLOSE
#  define DCLOSE(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DCLOSE(fmt, args...)
#endif

#ifdef ROCKET_DEBUG_IO
#  define DIO(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DIO(fmt, args...)
#endif

#ifdef ROCKET_DEBUG_WRITE
#  define DWRITE(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DWRITE(fmt, args...)
#endif

#ifdef ROCKET_DEBUG_READ
#  define DREAD(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DREAD(fmt, args...)
#endif

#ifdef ROCKET_DEBUG_INTR
#  define DINTR(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DINTR(fmt, args...)
#endif

#ifdef ROCKET_DEBUG_DELTA_MSR
#  define DMSR(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DMSR(fmt, args...)
#endif

#ifdef ROCKET_DEBUG_WAIT_UNTIL_SENT
#  define DWSENT(fmt, args...) printk(KERN_INFO "RP2: " fmt, ## args)
#else
#  define DWSENT(fmt, args...)
#endif

#endif // _ROCKET2_H
